/** 
 *  \file   	app.h
 *	\brief  	Definitions for DreamShell applications
 *	\date		2004-2011
 *	\author		SWAT
 *	\copyright	http://www.dc-swat.ru
 */

#ifndef _DS_APP_H
#define _DS_APP_H

#include <kos.h>
#include "list.h"
#include "gui.h"
#include "mxml.h"
#include "thread.h"
#include "console.h"
#include "lua.h"

#define APP_STATE_OPENED 0x00000001
#define APP_STATE_LOADED 0x00000002
#define APP_STATE_READY 0x00000004
#define APP_STATE_PROCESS 0x00000008
#define APP_STATE_SLEEP 0x00000020
#define APP_STATE_WAIT_UNLOAD 0x00000040

typedef struct App {
    
    const char *fn;
    const char *name;
    const char *icon;
    const char *ver;
    int id;
    
    Item_list_t *resources;
    Item_list_t *elements;
    mxml_node_t *xml;
    
    GUI_Widget *body;
    
    Thread_t *thd;
    lua_State *lua;
    
    int state;

} App_t;



int InitApps();
void ShutdownApps();

Item_list_t *GetAppList();
App_t *GetAppById(int id);
App_t *GetAppByFileName(const char *fn);
App_t *GetAppByName(const char *name);
App_t *GetCurApp();

App_t *AddApp(const char *fn);
int RemoveApp(App_t *app);

int OpenApp(App_t *app);
int CloseApp(App_t *app, int unload);
int SwitchApp(App_t *old_app, App_t *new_app, int unload);

void UnLoadOldApps();

int SetAppSleep(App_t *app, int sleep);

int LoadApp(App_t *app, int build);
int UnLoadApp(App_t *app);


int BuildAppBody(App_t *app);
int AddToAppBody(App_t *app, GUI_Widget *widget);
int RemoveFromAppBody(App_t *app, GUI_Widget *widget);

int CallAppBodyEvent(App_t *app, char *event);
void WaitApp(App_t *app);

// Utils

typedef struct AppSwitch {

	App_t *old_app;
	App_t *new_app;
	int unload;

} AppSwitch_t;

typedef struct lua_callback_data {

	lua_State *state;
	const char *script; 

} lua_callback_data_t;


typedef struct scrollbar_callback_data {

	GUI_Widget *scroll;
	GUI_Widget *panel; 

} scrollbar_callback_data_t;

char *FindXmlAttr(char *name, mxml_node_t *node, char *defValue);

void UnloadAppResources(Item_list_t *lst);

GUI_Surface *getElementSurface(App_t *app, char *name);
GUI_Font *getElementFont(App_t *app, mxml_node_t *node, char *name);
GUI_Surface *parseAppSurfaceRes(App_t *app, char *name, mxml_node_t *node, int w, int h);

void parseAppResource(App_t *app, mxml_node_t *node);
void parseAppResourceAfter(App_t *app, mxml_node_t *node);

uint32 GetAppExportFuncAddr(const char *fstr);
int CallAppExportFunc(const char *fstr);
GUI_Callback *CreateAppElementCallback(App_t *app, const char *event);

GUI_Widget *parseAppElement(App_t *app, mxml_node_t *node);
GUI_Widget *parseAppImageElement(App_t *app, mxml_node_t *node, char *name, int x, int y, int w, int h);
GUI_Widget *parseAppLabelElement(App_t *app, mxml_node_t *node, char *name, int x, int y, int w, int h);
GUI_Widget *parseAppButtonElement(App_t *app, mxml_node_t *node, char *name, int x, int y, int w, int h);
GUI_Widget *parseAppToggleButtonElement(App_t *app, mxml_node_t *node, char *name, int x, int y, int w, int h);
GUI_Widget *parseAppTextEntryElement(App_t *app, mxml_node_t *node, char *name, int x, int y, int w, int h);
GUI_Widget *parseAppScrollBarElement(App_t *app, mxml_node_t *node, char *name, int x, int y, int w, int h);
GUI_Widget *parseAppProgressBarElement(App_t *app, mxml_node_t *node, char *name, int x, int y, int w, int h);
GUI_Widget *parseAppPanelElement(App_t *app, mxml_node_t *node, char *name, int x, int y, int w, int h);
GUI_ListBox *parseAppListBoxElement(App_t *app, mxml_node_t *node, char *name, int x, int y, int w, int h);
GUI_Widget *parseAppCardStackElement(App_t *app, mxml_node_t *node, char *name, int x, int y, int w, int h);
GUI_Widget *parseAppRTFElement(App_t *app, mxml_node_t *node, char *name, int x, int y, int w, int h);
GUI_Widget *parseAppFileManagerElement(App_t *app, mxml_node_t *node, char *name, int x, int y, int w, int h);

#endif
