/** 
 *  \file   	console.h
 *	\brief  	Definitions for DreamShell console
 *	\date		2004-2011
 *	\author		SWAT
 *	\copyright	http://www.dc-swat.ru
 */


#ifndef _DS_CONSOLE_H
#define _DS_CONSOLE_H

#ifndef _NO_VIDEO_H
#include "video.h"
#else
#include "list.h"
#endif



typedef enum {
	CMD_OK = 0,			/* 0 */
	CMD_NO_ARG,         /* 1 */
	CMD_ERROR,		    /* 2 */
	CMD_NOT_EXISTS		/* 3 */
} CMD_RESULT;


typedef enum {
	CMD_TYPE_INTERNAL = 0,	/* 0 */
	CMD_TYPE_ELF,			/* 1 */
	CMD_TYPE_LUA,		    /* 2 */
	CMD_TYPE_DSC,		    /* 3 */
	CMD_TYPE_UKNOWN,		/* 4 */
} CMD_TYPES;


typedef int CmdHandler(int, char *[]);

typedef struct Cmd {

	const char *command;
	const char *desc;
	CmdHandler *handler;
	
} Cmd_t;



int CallCmd(int argc, char *argv[]);  
int CallExtCmd(int argc, char *argv[]); 
int CheckExtCmdType(const char *fn);

Cmd_t *AddCmd(const char *cmd, const char *helpmsg, CmdHandler *handler);
void RemoveCmd(Cmd_t *cmd);

Item_list_t *GetCmdList();
Cmd_t *GetCmdByName(const char *name);

int InitCmd(); 
void ShutdownCmd();


void SetConsoleDebug(int mode);


int ds_printf(const char *fmt, ...); 
int ds_uprintf(const char *fmt, ...);
int dbg_printf(const char *fmt, ...);


int dsystem(const char *buff); 
int dsystemf(const char *fmt, ...);
int dsystem_script(const char *fn);
int dsystem_buff(const char *buff);

int InitConsole(const char *font, const char *background, int lines, int x, int y, int w, int h, int alpha);
void ShutdownConsole();

void DrawConsole();
int ToggleConsole();
void ShowConsole();
void HideConsole();
int ConsoleIsVisible();

#ifndef _NO_VIDEO_H
ConsoleInformation *GetConsole();
#endif

extern dbgio_handler_t dbgio_ds;
extern dbgio_handler_t dbgio_sd;


#endif
