/**
 * Copyright (c) 2011 by SWAT <swat@211.ru>
 *
 * This file is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef SD_MMC_H
#define SD_MMC_H

#include <arch/types.h>

/**
 * \file
 * SDC/MMC driver for SPI
 *
 * \author SWAT
 */

#define SD_MMC_IDLE_TOKEN 0xff
#define SD_MMC_BUSY_TOKEN 0
#define SD_MMC_BLOCK_SIZE_DEFAULT 512
#define SD_MMC_COMMAND_RESPONSE_START 7

/**
 * @name Commands
 * @{
 */

#define SD_MMC_CMD_GO_IDLE_STATE 0
#define SD_MMC_CMD_SEND_OP_COND 1
#define SD_MMC_CMD_SEND_IF_COND 8
#define SD_MMC_CMD_SEND_CSD 9
#define SD_MMC_CMD_SEND_CID 10
#define SD_MMC_CMD_STOP_TRANSMISSION 12
#define SD_MMC_CMD_SEND_STATUS 13
#define SD_MMC_CMD_SET_BLOCKLEN 16
#define SD_MMC_CMD_READ_SINGLE_BLOCK 17
#define SD_MMC_CMD_READ_MULTIPLE_BLOCK 18
#define SD_MMC_CMD_SET_BLOCK_COUNT 23
#define SD_MMC_CMD_WRITE_BLOCK 24
#define SD_MMC_CMD_WRITE_MULTIPLE_BLOCK 25
#define SD_MMC_CMD_PROGRAM_CSD 27
#define SD_MMC_CMD_SET_WRITE_PROT 28
#define SD_MMC_CMD_CLR_WRITE_PROT 29
#define SD_MMC_CMD_SEND_WRITE_PROT 30
#define SD_MMC_CMD_TAG_SECTOR_START 32
#define SD_MMC_CMD_TAG_SECTOR_END 33
#define SD_MMC_CMD_UNTAG_SECTOR 34
#define SD_MMC_CMD_TAG_ERASE_GROUP_START 35
#define SD_MMC_CMD_TAG_ERASE_GROUP_END 36
#define SD_MMC_CMD_UNTAG_ERASE_GROUP 37
#define SD_MMC_CMD_ERASE 38
#define SD_MMC_CMD_LOCK_UNLOCK 42
#define SD_MMC_CMD_APP_CMD 55
#define SD_MMC_CMD_GEN_CND 56
#define SD_MMC_CMD_READ_OCR 58
#define SD_MMC_CMD_CRC_ON_OFF 59

/** @} */

/**
 * @name Application Commands
 * @{
 */

#define SD_MMC_ACMD_SD_SEND_OP_COND 41

/** @} */

/**
 * @name Command Flags
 * @{
 */

#define SD_MMC_FLAG_HCS 0x40000000U
#define SD_MMC_FLAG_VHS_2_7_TO_3_3 0x00000100U
#define SD_MMC_FLAG_CHECK_PATTERN 0x000000aaU

/** @} */

/**
 * @name Command Fields
 * @{
 */

#define SD_MMC_COMMAND_SET_COMMAND(c, cmd) (c) [1] = (uint8_t) (0x40 + ((cmd) & 0x3f))
#define SD_MMC_COMMAND_SET_ARGUMENT(c, arg) SD_MMC_put_uint32( (arg), &((c) [2]))
#define SD_MMC_COMMAND_SET_CRC7(c, crc7) ((c) [6] = ((crc7) << 1) | 1U)
#define SD_MMC_COMMAND_GET_CRC7(c) ((c) [6] >> 1)

/** @} */

/**
 * @name Response Fields
 * @{
 */

#define SD_MMC_IS_RESPONSE( r) (((r) & 0x80) == 0)
#define SD_MMC_IS_ERRORLESS_RESPONSE( r) (((r) & 0x7e) == 0)
#define SD_MMC_IS_NOT_IDLE_RESPONSE( r) (((r) & 0x81) == 0)
#define SD_MMC_IS_DATA_ERROR( r) (((r) & 0xe0) == 0)
#define SD_MMC_IS_DATA_REJECTED( r) (((r) & 0x1f) != 0x05)

/** @} */

/**
 * @name Card Identification
 * @{
 */

#define SD_MMC_CID_SIZE 16

#define SD_MMC_CID_GET_MID( cid) ((cid) [0])
#define SD_MMC_CID_GET_OID( cid) SD_MMC_get_uint16( cid + 1)
#define SD_MMC_CID_GET_PNM( cid, i) ((char) (cid) [3 + (i)])
#define SD_MMC_CID_GET_PRV( cid) ((cid) [9])
#define SD_MMC_CID_GET_PSN( cid) SD_MMC_get_uint32( cid + 10)
#define SD_MMC_CID_GET_MDT( cid) ((cid) [14])
#define SD_MMC_CID_GET_CRC7( cid) ((cid) [15] >> 1)

/** @} */

/**
 * @name Card Specific Data
 * @{
 */

#define SD_MMC_CSD_SIZE 16

#define SD_MMC_CSD_GET_CSD_STRUCTURE( csd) ((csd) [0] >> 6)
#define SD_MMC_CSD_GET_SPEC_VERS( csd) (((csd) [0] >> 2) & 0xf)
#define SD_MMC_CSD_GET_TAAC( csd) ((csd) [1])
#define SD_MMC_CSD_GET_NSAC( csd) ((uint32_t) (csd) [2])
#define SD_MMC_CSD_GET_TRAN_SPEED( csd) ((csd) [3])
#define SD_MMC_CSD_GET_C_SIZE( csd) ((((uint32_t) (csd) [6] & 0x3) << 10) + (((uint32_t) (csd) [7]) << 2) + ((((uint32_t) (csd) [8]) >> 6) & 0x3))
#define SD_MMC_CSD_GET_C_SIZE_MULT( csd) ((((csd) [9] & 0x3) << 1) + (((csd) [10] >> 7) & 0x1))
#define SD_MMC_CSD_GET_READ_BLK_LEN( csd) ((uint32_t) (csd) [5] & 0xf)
#define SD_MMC_CSD_GET_WRITE_BLK_LEN( csd) ((((uint32_t) (csd) [12] & 0x3) << 2) + ((((uint32_t) (csd) [13]) >> 6) & 0x3))
#define SD_MMC_CSD_1_GET_C_SIZE( csd) ((((uint32_t) (csd) [7] & 0x3f) << 16) + (((uint32_t) (csd) [8]) << 8) + (uint32_t) (csd) [9])

/** @} */

#define SD_MMC_INVALIDATE_RESPONSE_INDEX( e) e->response_index = SD_MMC_COMMAND_SIZE

/**
 * @name Data Start and Stop Tokens
 * @{
 */

#define SD_MMC_START_BLOCK_SINGLE_BLOCK_READ 0xfe
#define SD_MMC_START_BLOCK_MULTIPLE_BLOCK_READ 0xfe
#define SD_MMC_START_BLOCK_SINGLE_BLOCK_WRITE 0xfe
#define SD_MMC_START_BLOCK_MULTIPLE_BLOCK_WRITE 0xfc
#define SD_MMC_STOP_TRANSFER_MULTIPLE_BLOCK_WRITE 0xfd

/** @} */



/**
 * 1 idle token before command
 * 6 bytes for the command
 * 1 to 8 bytes for response start (N_CR)
 * 1 to 2 bytes for response
 * 1 idle token after command (minimum N_RC)
 */
#define SD_MMC_COMMAND_SIZE 18

#define SD_MMC_TRANSFER_MODE_DEFAULT { .baudrate = 400000, .bits_per_char = 8, .lsb_first = FALSE, .clock_inv = FALSE, .clock_phs = FALSE, .idle_char = SD_MMC_IDLE_TOKEN }

#define SD_MMC_COMMAND_DEFAULT { \
	SD_MMC_IDLE_TOKEN, \
	0x40, 0, 0, 0, 0, 0x95, \
	SD_MMC_IDLE_TOKEN, SD_MMC_IDLE_TOKEN, \
	SD_MMC_IDLE_TOKEN, SD_MMC_IDLE_TOKEN, \
	SD_MMC_IDLE_TOKEN, SD_MMC_IDLE_TOKEN, \
	SD_MMC_IDLE_TOKEN, SD_MMC_IDLE_TOKEN, \
	SD_MMC_IDLE_TOKEN, SD_MMC_IDLE_TOKEN, \
	SD_MMC_IDLE_TOKEN \
}

#define SD_MMC_N_AC_MAX_DEFAULT 8

/*
typedef struct spi_transfer_mode {
	
	uint32_t 	baudrate;
	uint8_t 	bits_per_char;
	int 		lsb_first;
	int 		clock_inv;
	int 		clock_phs;
	uint32_t 	idle_char;
	
} spi_transfer_mode_t;


typedef struct {
	const char *device_name;
	int bus;
	spi_transfer_mode_t transfer_mode;
	uint8_t command[SD_MMC_COMMAND_SIZE];
	uint8_t response[SD_MMC_COMMAND_SIZE];
	int response_index;
	uint32_t n_ac_max;
	uint32_t block_number;
	uint32_t block_size;
	uint32_t block_size_shift;
	int busy;
	int verbose;
	int schedule_if_busy;
} SD_MMC_driver_entry;

extern SD_MMC_driver_entry SD_MMC_driver_table [];
extern size_t SD_MMC_driver_table_size;
*/

#endif /* SD_MMC_H */
