/**
 * Copyright (c) 2011 by SWAT <swat@211.ru>
 *
 * This file is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#ifndef _SPI_H
#define _SPI_H

#include <arch/types.h>

/**
 * \file
 * SPI interface for Sega Dreamcast serial port
 *
 * \author SWAT
 */


#define SPI_MAX_SCIF_SPEED		8000000
#define SPI_MAX_GPIO_SPEED		20000000

/**
 * Max speed.
 */
#define SPI_DEFAULT_DELAY		-1

#define SPI_SDC_MMC_DELAY		SPI_DEFAULT_DELAY
#define SPI_SDC_MMC_CMD_DELAY	10
#define SPI_ENC28J60_DELAY		5


/*
 * Chip select pins
 */
#define SPI_CS_SCIF_RTS 	10
#define SPI_CS_GPIO_VGA 	8	// This is a real GPIO index
#define SPI_CS_GPIO_RGB 	9

#define SPI_CS_SDC 			SPI_CS_SCIF_RTS
#define SPI_CS_ENC28J60 	SPI_CS_GPIO_VGA
#define SPI_CS_SDC_2 		SPI_CS_GPIO_RGB


/**
 * Initializes the SPI interface
 */
void spi_init();

/**
 * Shutdown the SPI interface
 */
void spi_shutdown();

/**
 * Change the SPI delay for clock
 *
 * \param[in] The SPI delay
 */
void spi_set_delay(int delay);

/**
 * Getting current SPI delay
 *
 * \returns The SPI delay
 */
int spi_get_delay();

/**
 * Switch on CS pin
 *
 * \param[in] the CS pin
 */
void spi_cs_on(int cs);

/**
 * Switch off CS pin
 *
 * \param[in] the CS pin
 */
void spi_cs_off(int cs);

/**
 * Sends a byte over the SPI bus.
 *
 * \param[in] b The byte to send.
 */

void spi_send_byte(register uint8 b);

/**
 * Receives a byte from the SPI bus.
 *
 * \returns The received byte.
 */
uint8 spi_rec_byte();

/**
 * Send and receive a byte to/from the SPI bus.
 *
 * \param[in] b The byte to send.
 * \returns The received byte.
 */
uint8 spi_sr_byte(register uint8 b);

/**
 * Slow send and receive a byte to/from the SPI bus. 
 * Used for SDC/MMC commands.
 *
 * \param[in] b The byte to send.
 * \returns The received byte.
 */
uint8 spi_slow_sr_byte(register uint8 b);

/**
 * Sends data contained in a buffer over the SPI bus.
 *
 * \param[in] data A pointer to the buffer which contains the data to send.
 * \param[in] len The number of bytes to send.
 */
void spi_send_data(const uint8* data, uint16 data_len);

/**
 * Receives multiple bytes from the SPI bus and writes them to a buffer.
 *
 * \param[out] buffer A pointer to the buffer into which the data gets written.
 * \param[in] len The number of bytes to read.
 */
void spi_rec_data(uint8* buffer, uint16 buffer_len);


#endif /* _SPI_H */
