/** 
 *  \file   	net.h
 *	\brief  	Definitions for DreamShell network
 *	\date		2004-2011
 *	\author		SWAT
 *	\copyright	http://www.dc-swat.ru
 */   

#ifndef _DS_NET_H
#define _DS_NET_H


#ifdef DS_ARCH_DC

#include <sys/cdefs.h>
#include <arch/types.h>
#include <sys/queue.h>

#ifndef USE_LWIP

#include <kos/net.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netinet/in.h>

#else

#ifndef __KOS_NET_H

typedef struct knetif {
    LIST_ENTRY(knetif)  if_list;
    const char          *name;
    const char          *descr;
    int                 index;
    uint32              dev_id;
    uint32              flags;
    uint8               mac_addr[6];
    uint8               ip_addr[4];
    uint8               netmask[4];
    uint8               gateway[4];
    uint8               broadcast[4];
    uint8               dns[4];
    int                 mtu;
    struct in6_addr     ip6_lladdr;
    struct in6_addr     *ip6_addrs;
    int                 ip6_addr_count;
    struct in6_addr     ip6_gateway;
    uint32              mtu6;
    int                 hop_limit;
    int (*if_detect)(struct knetif * self);
    int (*if_init)(struct knetif * self);
    int (*if_shutdown)(struct knetif * self);
    int (*if_start)(struct knetif * self);
    int (*if_stop)(struct knetif * self);
    int (*if_tx)(struct knetif * self, const uint8 * data, int len, int blocking);
    int (*if_tx_commit)(struct knetif * self);
    int (*if_rx_poll)(struct knetif * self);
    int (*if_set_flags)(struct knetif * self, uint32 flags_and, uint32 flags_or);
    int (*if_set_mc)(struct knetif *self, const uint8 *list, int count);
} netif_t;

extern struct netif_list net_if_list;
struct netif_list * net_get_if_list();
extern netif_t *net_default_dev;
netif_t *net_set_default(netif_t *n);
int net_reg_device(netif_t *device);
int net_unreg_device(netif_t *device);
int net_init();
void net_shutdown();
int net_dev_init();

#endif

#include "lwip/lwip.h"
#include "lwip/sockets.h"

#endif


#elif DS_ARCH_PSP

#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netinet/tcp.h>
#include <sys/fd_set.h>



#else

#include <winsock.h>


#endif



#ifdef DS_ARCH_DC

/***   General network stuff   ***/


/***   Filesystem   ***/
/*
int tcpfs_init();
void tcpfs_shutdown();

int httpfs_init();
void httpfs_shutdown();
*/

#endif

/***   Net utils   ***/

#define READIP(dst, src) \
        dst = ((src[0]) << 24) | \
                ((src[1]) << 16) | \
                ((src[2]) << 8) | \
                ((src[3]) << 0)
                
                
#define SETIP(target, src) \
	IP4_ADDR(target, \
		(src >> 24) & 0xff, \
		(src >> 16) & 0xff, \
		(src >> 8) & 0xff, \
		(src >> 0) & 0xff)
              
 



#endif
